import Node
import unittest

class Owner(object):
    child = Node.NodeReference()

class OwnerMap(object):
    children = Node.NodeMap()

class OwnerList(object):
    children = Node.NodeList()

class Child(Node.Node):
    pass

class UnitTest(unittest.TestCase):

    def testNode(self):
        owner = Owner()
        child1 = Child()
        child2 = Child()
        self.assertEqual(list(child1.owners),[])
        self.assertEqual(list(child2.owners),[])
        self.assertEqual(owner.child,None)

        owner.child = child1
        self.assertEqual(list(child1.owners), [owner])
        self.assertEqual(list(child2.owners), [])
        self.assertEqual(owner.child, child1)

        owner.child = child2
        self.assertEqual(list(child1.owners), [])
        self.assertEqual(list(child2.owners), [owner])
        self.assertEqual(owner.child, child2)

        owner.child = None
        self.assertEqual(list(child1.owners),[])
        self.assertEqual(list(child2.owners),[])
        self.assertEqual(owner.child,None)
        
        owner.child = child1
        self.assertEqual(list(child1.owners), [owner])
        owner = None
        self.assertEqual(list(child1.owners),[])
        self.assertEqual(owner, None)


    def testNodeMap(self):
        owner = OwnerMap()
        child = Child()
        owner.children['a'] = child
        self.assertEqual(list(child.owners),[owner])
        del owner.children['a']
        self.assertEqual(list(child.owners),[])

        child2 = Child()
        owner.children.setdefault('a',child)
        owner.children['b'] = child2
        self.assertEqual(list(child.owners),[owner])
        self.assertEqual(list(child2.owners),[owner])
        owner.children['a'] = owner.children['b']
        self.assertEqual(list(child.owners),[])
        self.assertEqual(list(child2.owners),[owner,owner])
        del owner.children['a']
        self.assertEqual(list(child2.owners),[owner])
        owner.children.clear()
        self.assertEqual(list(child2.owners),[])

        owner.children = dict(a=child, b=child2)
        self.assertEqual(list(owner.children.pop('a').owners), [])
        self.assertEqual(list(child2.owners), [owner])
        self.assertEqual(owner.children.popitem(),('b',child2))
        self.assertEqual(list(child2.owners), [])

        owner.children['a'] = child
        owner.children.update(b=child2)
        self.assertEqual(list(child2.owners), [owner])
        self.assertEqual(list(child.owners), [owner])
        owner.children.update(a=child2)
        self.assertEqual(list(child.owners), [])
        self.assertEqual(list(child2.owners),[owner,owner])

        del owner
        self.assertEqual(list(child2.owners), [])


    def testNodeList(self):
        owner = OwnerList()
        child1 = Child()
        child2 = Child()

        owner.children.append(child1)
        owner.children.append(child2)
        self.assertEqual(owner.children, [child1,child2])
        self.assertEqual(list(child1.owners),[owner])
        self.assertEqual(list(child2.owners),[owner])

        owner.children.extend([child1,child2])
        self.assertEqual(owner.children, [child1,child2,child1,child2])
        self.assertEqual(list(child1.owners),[owner,owner])
        self.assertEqual(list(child2.owners),[owner,owner])

        self.assertRaises(ValueError, self._testNodeList_1, owner)
        self.assertEqual(owner.children, [child1,child2,child1,child2])
        self.assertEqual(list(child1.owners),[owner,owner])
        self.assertEqual(list(child2.owners),[owner,owner])

        owner.children[1:2] = []
        self.assertEqual(list(child1.owners), [owner,owner])
        self.assertEqual(list(child2.owners), [owner])
        
        owner.children[:] = []
        self.assertEqual(list(child1.owners), [])
        self.assertEqual(list(child2.owners), [])

        owner.children[:] = [child1,child2]
        self.assertEqual(list(child1.owners),[owner])
        self.assertEqual(list(child2.owners),[owner])

        del owner.children[0]
        self.assertEqual(list(child1.owners),[])
        self.assertEqual(list(child2.owners),[owner])

        del owner.children[:]
        self.assertEqual(list(child2.owners),[])

        owner.children = [child1,child1]
        owner.children.insert(1,child2)
        self.assertEqual(owner.children,[child1,child2,child1])
        self.assertEqual(list(child1.owners), [owner,owner])
        self.assertEqual(list(child2.owners), [owner])

        self.assertEqual(list(owner.children.pop().owners), [owner])
        self.assertEqual(owner.children.pop(), child2)
        self.assertEqual(owner.children, [child1])

        owner.children.append(child2)
        self.assertEqual(list(owner.children.pop(0).owners), [])
        self.assertEqual(owner.children, [child2])
        owner.children.append(child1)
        owner.children.remove(child2)
        self.assertEqual(list(child1.owners),[owner])
        self.assertEqual(list(child2.owners),[])
        self.assertEqual(owner.children, [child1])

        del owner
        self.assertEqual(list(child1.owners), [])
        

    def _testNodeList_1(self, owner):
        owner.children[::2] = []
        

if __name__ == '__main__':
    unittest.main()
